@@ -35,6 +35,7 @@
 #include <sys/stat.h>
 
 #include "arg_parser.h"
+#include "block.h"
 #include "ddrescue.h"
 
 
@@ -65,9 +66,10 @@ void show_help( const int cluster, const int hardbs ) throw()
   std::printf( "  -F, --fill=<types>           fill given type areas with infile data (?*/-+)\n" );
   std::printf( "  -g, --generate-logfile       generate approximate logfile from partial copy\n" );
   std::printf( "  -i, --input-position=<pos>   starting position in input file [0]\n" );
+  std::printf( "  -m, --domain-logfile=<file>  restrict domain to areas marked finished in file\n" );
   std::printf( "  -n, --no-split               do not try to split or retry error areas\n" );
   std::printf( "  -o, --output-position=<pos>  starting position in output file [ipos]\n" );
-  std::printf( "  -q, --quiet                  quiet operation\n" );
+  std::printf( "  -q, --quiet                  suppress all messages\n" );
   std::printf( "  -r, --max-retries=<n>        exit after given retries (-1=infinity) [0]\n" );
   std::printf( "  -R, --retrim                 mark all error areas as non-trimmed\n" );
   std::printf( "  -s, --max-size=<bytes>       maximum size of data to be copied\n" );
@@ -90,17 +92,16 @@ void show_version() throw()
   }
 
 
-long long getnum( const char * ptr, const int bs, const int verbosity,
-                  const long long min = LONG_LONG_MIN + 1,
-                  const long long max = LONG_LONG_MAX ) throw()
+long long getnum( const char * ptr, const int bs,
+                  const long long min = LLONG_MIN + 1,
+                  const long long max = LLONG_MAX ) throw()
   {
   errno = 0;
   char *tail;
   long long result = strtoll( ptr, &tail, 0 );
   if( tail == ptr )
     {
-    if( verbosity >= 0 )
-      show_error( "bad or missing numerical argument", 0, true );
+    show_error( "bad or missing numerical argument", 0, true );
     std::exit( 1 );
     }
 
@@ -130,27 +131,26 @@ long long getnum( const char * ptr, const int bs, const int verbosity,
       }
     if( bad_multiplier )
       {
-      if( verbosity >= 0 )
-        show_error( "bad multiplier in numerical argument", 0, true );
+      show_error( "bad multiplier in numerical argument", 0, true );
       std::exit( 1 );
       }
     for( int i = 0; i < exponent; ++i )
       {
-      if( LONG_LONG_MAX / factor >= llabs( result ) ) result *= factor;
+      if( LLONG_MAX / factor >= llabs( result ) ) result *= factor;
       else { errno = ERANGE; break; }
       }
     }
   if( !errno && ( result < min || result > max ) ) errno = ERANGE;
   if( errno )
     {
-    if( verbosity >= 0 ) show_error( "numerical argument out of limits" );
+    show_error( "numerical argument out of limits" );
     std::exit( 1 );
     }
   return result;
   }
 
 
-void check_fill_types( const std::string filltypes, const int verbosity ) throw()
+void check_fill_types( const std::string filltypes ) throw()
   {
   bool good = true;
   for( unsigned int i = 0; i < filltypes.size(); ++i )
@@ -158,7 +158,7 @@ void check_fill_types( const std::string filltypes, const int verbosity ) throw(
       { good = false; break; }
   if( !filltypes.size() || !good )
     {
-    if( verbosity >= 0 ) show_error( "invalid type for `fill' option" );
+    show_error( "invalid type for `fill' option" );
     std::exit( 1 );
     }
   }
@@ -173,41 +173,32 @@ bool check_identical( const char * name1, const char * name2 ) throw()
   }
 
 
-int do_fill( long long ipos, const long long opos, const long long max_size,
+int do_fill( long long ipos, const long long opos, Domain & domain,
              const char *iname, const char *oname, const char *logname,
-             const int cluster, const int hardbs, const int verbosity,
+             const int cluster, const int hardbs,
              const std::string & filltypes, const bool synchronous ) throw()
   {
   if( !logname )
     {
-    if( verbosity >= 0 )
-      show_error( "logfile required in fill mode", 0, true );
+    show_error( "logfile required in fill mode", 0, true );
     return 1;
     }
 
-  Fillbook fillbook( ipos, opos, max_size, logname, cluster, hardbs,
-                     verbosity, synchronous );
+  Fillbook fillbook( ipos, opos, domain, logname, cluster, hardbs, synchronous );
   if( fillbook.domain().size() == 0 )
-    { if( verbosity >= 0 ) { show_error( "Nothing to do" ); } return 0; }
+    { show_error( "Nothing to do" ); return 0; }
 
   const int ides = open( iname, O_RDONLY );
   if( ides < 0 )
-    { if( verbosity >= 0 ) show_error( "cannot open input file", errno );
-      return 1; }
+    { show_error( "cannot open input file", errno ); return 1; }
   if( !fillbook.read_buffer( ides ) )
-    {
-    if( verbosity >= 0 )
-      show_error( "error reading fill data from input file" );
-    return 1;
-    }
+    { show_error( "error reading fill data from input file" ); return 1; }
 
   const int odes = open( oname, O_WRONLY | O_CREAT, 0644 );
   if( odes < 0 )
-    { if( verbosity >= 0 ) show_error( "cannot open output file", errno );
-      return 1; }
+    { show_error( "cannot open output file", errno ); return 1; }
   if( lseek( odes, 0, SEEK_SET ) )
-    { if( verbosity >= 0 ) show_error( "output file is not seekable" );
-      return 1; }
+    { show_error( "output file is not seekable" ); return 1; }
 
   if( verbosity >= 0 ) std::printf( "\n\n" );
   if( verbosity > 0 )
@@ -229,43 +220,36 @@ int do_fill( long long ipos, const long long opos, const long long max_size,
   }
 
 
-int do_generate( const long long ipos, const long long opos, const long long max_size,
+int do_generate( const long long ipos, const long long opos, Domain & domain,
                  const char *iname, const char *oname, const char *logname,
-                 const int cluster, const int hardbs, const int verbosity ) throw()
+                 const int cluster, const int hardbs ) throw()
   {
   if( !logname )
     {
-    if( verbosity >= 0 )
-      show_error( "logfile must be specified in generate-logfile mode", 0, true );
+    show_error( "logfile must be specified in generate-logfile mode", 0, true );
     return 1;
     }
   const int ides = open( iname, O_RDONLY );
   if( ides < 0 )
-    { if( verbosity >= 0 ) show_error( "cannot open input file", errno );
-      return 1; }
+    { show_error( "cannot open input file", errno ); return 1; }
   const long long isize = lseek( ides, 0, SEEK_END );
   if( isize < 0 )
-    { if( verbosity >= 0 ) show_error( "input file is not seekable" );
-      return 1; }
+    { show_error( "input file is not seekable" ); return 1; }
 
-  Rescuebook genbook( ipos, opos, max_size, isize, logname, cluster, hardbs,
-                      verbosity );
+  Rescuebook genbook( ipos, opos, domain, isize, logname, cluster, hardbs );
   if( genbook.domain().size() == 0 )
-    { if( verbosity >= 0 ) { show_error( "Nothing to do" ); } return 0; }
+    { show_error( "Nothing to do" ); return 0; }
   if( !genbook.blank() && genbook.current_status() != Logbook::generating )
     {
-    if( verbosity >= 0 )
-      show_error( "logfile alredy exists and is non-empty", 0, true );
+    show_error( "logfile alredy exists and is non-empty", 0, true );
     return 1;
     }
 
   const int odes = open( oname, O_RDONLY );
   if( odes < 0 )
-    { if( verbosity >= 0 ) show_error( "cannot open output file", errno );
-      return 1; }
+    { show_error( "cannot open output file", errno ); return 1; }
   if( lseek( odes, 0, SEEK_SET ) )
-    { if( verbosity >= 0 ) show_error( "output file is not seekable" );
-      return 1; }
+    { show_error( "output file is not seekable" ); return 1; }
 
   if( verbosity >= 0 ) std::printf( "\n\n" );
   if( verbosity > 0 )
@@ -284,42 +268,37 @@ int do_generate( const long long ipos, const long long opos, const long long max
   }
 
 
-int do_rescue( const long long ipos, const long long opos, const long long max_size,
+int do_rescue( const long long ipos, const long long opos, Domain & domain,
                const char *iname, const char *oname, const char *logname,
                const int cluster, const int hardbs,
                const int max_errors, const int max_retries,
-               const int o_direct, const int o_trunc, const int verbosity,
+               const int o_direct, const int o_trunc,
                const bool complete_only, const bool nosplit, const bool retrim,
                const bool sparse, const bool synchronous ) throw()
   {
   const int ides = open( iname, O_RDONLY | o_direct );
   if( ides < 0 )
-    { if( verbosity >= 0 ) show_error( "cannot open input file", errno );
-      return 1; }
+    { show_error( "cannot open input file", errno ); return 1; }
   const long long isize = lseek( ides, 0, SEEK_END );
   if( isize < 0 )
-    { if( verbosity >= 0 ) show_error( "input file is not seekable" );
-      return 1; }
+    { show_error( "input file is not seekable" ); return 1; }
 
-  Rescuebook rescuebook( ipos, opos, max_size, isize, logname, cluster, hardbs,
-                         verbosity, max_errors, max_retries, complete_only,
+  Rescuebook rescuebook( ipos, opos, domain, isize, logname, cluster, hardbs,
+                         max_errors, max_retries, complete_only,
                          nosplit, retrim, sparse, synchronous );
   if( rescuebook.domain().size() == 0 )
-    { if( verbosity >= 0 ) { show_error( "Nothing to do" ); } return 0; }
+    { show_error( "Nothing to do" ); return 0; }
   if( o_trunc && !rescuebook.blank() )
     {
-    if( verbosity >= 0 )
-      show_error( "outfile truncation and logfile input are incompatible", 0, true );
+    show_error( "outfile truncation and logfile input are incompatible", 0, true );
     return 1;
     }
 
   const int odes = open( oname, O_WRONLY | O_CREAT | o_trunc, 0644 );
   if( odes < 0 )
-    { if( verbosity >= 0 ) show_error( "cannot open output file", errno );
-      return 1; }
+    { show_error( "cannot open output file", errno ); return 1; }
   if( lseek( odes, 0, SEEK_SET ) )
-    { if( verbosity >= 0 ) show_error( "output file is not seekable" );
-      return 1; }
+    { show_error( "output file is not seekable" ); return 1; }
 
   if( verbosity >= 0 ) std::printf( "\n\n" );
   if( verbosity > 0 )
@@ -353,25 +332,30 @@ int do_rescue( const long long ipos, const long long opos, const long long max_s
 } // end namespace
 
 
-void internal_error( const char * msg ) throw()
-  {
-  char buf[80];
-  snprintf( buf, sizeof( buf ), "internal error: %s", msg );
-  show_error( buf );
-  std::exit( 3 );
-  }
+int verbosity = 0;
 
 
 void show_error( const char * msg, const int errcode, const bool help ) throw()
   {
-  if( msg && msg[0] != 0 )
+  if( verbosity >= 0 )
     {
-    std::fprintf( stderr, "%s: %s", program_name, msg );
-    if( errcode > 0 ) std::fprintf( stderr, ": %s", strerror( errcode ) );
-    std::fprintf( stderr, "\n" );
+    if( msg && msg[0] != 0 )
+      {
+      std::fprintf( stderr, "%s: %s", program_name, msg );
+      if( errcode > 0 ) std::fprintf( stderr, ": %s", strerror( errcode ) );
+      std::fprintf( stderr, "\n" );
+      }
+    if( help && invocation_name && invocation_name[0] != 0 )
+      std::fprintf( stderr, "Try `%s --help' for more information.\n", invocation_name );
     }
-  if( help && invocation_name && invocation_name[0] != 0 )
-    std::fprintf( stderr, "Try `%s --help' for more information.\n", invocation_name );
+  }
+
+
+void internal_error( const char * msg ) throw()
+  {
+  std::string s( "internal error: " ); s += msg;
+  show_error( s.c_str() );
+  std::exit( 3 );
   }
 
 
@@ -385,10 +369,11 @@ void write_logfile_header( FILE * f ) throw()
 int main( const int argc, const char * argv[] ) throw()
   {
   long long ipos = 0, opos = -1, max_size = -1;
+  const char * domain_logfile_name = 0;
   const int cluster_bytes = 65536, default_hardbs = 512;
   int cluster = 0, hardbs = 512;
   int max_errors = -1, max_retries = 0;
-  int o_direct = 0, o_trunc = 0, verbosity = 0;
+  int o_direct = 0, o_trunc = 0;
   bool complete_only = false, generate = false, nosplit = false;
   bool retrim = false, sparse = false, synchronous = false;
   std::string filltypes;
@@ -407,6 +392,7 @@ int main( const int argc, const char * argv[] ) throw()
     { 'g', "generate-logfile", Arg_parser::no  },
     { 'h', "help",             Arg_parser::no  },
     { 'i', "input-position",   Arg_parser::yes },
+    { 'm', "domain-logfile",   Arg_parser::yes },
     { 'n', "no-split",         Arg_parser::no  },
     { 'o', "output-position",  Arg_parser::yes },
     { 'q', "quiet",            Arg_parser::no  },
@@ -431,31 +417,30 @@ int main( const int argc, const char * argv[] ) throw()
     const char * arg = parser.argument( argind ).c_str();
     switch( code )
       {
-      case 'b': hardbs = getnum( arg, 0, verbosity, 1, INT_MAX ); break;
+      case 'b': hardbs = getnum( arg, 0, 1, INT_MAX ); break;
       case 'B': format_num( 0, 0, -1 ); break;		// set binary prefixes
-      case 'c': cluster = getnum( arg, 1, verbosity, 1, INT_MAX ); break;
+      case 'c': cluster = getnum( arg, 1, 1, INT_MAX ); break;
       case 'C': complete_only = true; break;
       case 'd':
 #ifdef O_DIRECT
                 o_direct = O_DIRECT;
 #endif
                 if( !o_direct )
-                  { if( verbosity >= 0 )
-                      show_error( "direct disc access not available" );
-                    return 1; }
+                  { show_error( "direct disc access not available" ); return 1; }
                 break;
       case 'D': synchronous = true; break;
-      case 'e': max_errors = getnum( arg, 0, verbosity, -1, INT_MAX ); break;
-      case 'F': filltypes = arg; check_fill_types( filltypes, verbosity ); break;
+      case 'e': max_errors = getnum( arg, 0, -1, INT_MAX ); break;
+      case 'F': filltypes = arg; check_fill_types( filltypes ); break;
       case 'g': generate = true; break;
       case 'h': show_help( cluster_bytes / default_hardbs, default_hardbs ); return 0;
-      case 'i': ipos = getnum( arg, hardbs, verbosity, 0 ); break;
+      case 'i': ipos = getnum( arg, hardbs, 0 ); break;
+      case 'm': domain_logfile_name = arg; break;
       case 'n': nosplit = true; break;
-      case 'o': opos = getnum( arg, hardbs, verbosity, 0 ); break;
+      case 'o': opos = getnum( arg, hardbs, 0 ); break;
       case 'q': verbosity = -1; break;
-      case 'r': max_retries = getnum( arg, 0, verbosity, -1, INT_MAX ); break;
+      case 'r': max_retries = getnum( arg, 0, -1, INT_MAX ); break;
       case 'R': retrim = true; break;
-      case 's': max_size = getnum( arg, hardbs, verbosity, -1 ); break;
+      case 's': max_size = getnum( arg, hardbs, -1 ); break;
       case 'S': sparse = true; break;
       case 't': o_trunc = O_TRUNC; break;
       case 'v': verbosity = 1; break;
@@ -475,43 +460,38 @@ int main( const int argc, const char * argv[] ) throw()
   if( argind < parser.arguments() ) oname = parser.argument( argind++ ).c_str();
   if( argind < parser.arguments() ) logname = parser.argument( argind++ ).c_str();
   if( argind < parser.arguments() )
-    { if( verbosity >= 0 ) show_error( "too many files", 0, true );
-      return 1; }
+    { show_error( "too many files", 0, true ); return 1; }
 
   // end scan arguments
 
   if( !iname || !oname )
     {
-    if( verbosity >= 0 )
-      show_error( "both input and output files must be specified", 0, true );
+    show_error( "both input and output files must be specified", 0, true );
     return 1;
     }
   if( check_identical ( iname, oname ) )
-    { if( verbosity >= 0 ) show_error( "infile and outfile are the same" );
-      return 1; }
+    { show_error( "infile and outfile are the same" ); return 1; }
+
+  Domain domain( domain_logfile_name, ipos, max_size );
 
   if( filltypes.size() )
     {
-    if( verbosity >= 0 &&
-        ( max_errors >= 0 || max_retries || o_direct || o_trunc ||
-          complete_only || generate || nosplit || retrim || sparse || synchronous ) )
+    if( max_errors >= 0 || max_retries || o_direct || o_trunc ||
+        complete_only || generate || nosplit || retrim || sparse || synchronous )
       show_error( "warning: options -C -d -D -e -g -n -r -R -S and -t are ignored in fill mode" );
 
-    return do_fill( ipos, opos, max_size, iname, oname, logname, cluster,
-                    hardbs, verbosity, filltypes, synchronous );
+    return do_fill( ipos, opos, domain, iname, oname, logname, cluster,
+                    hardbs, filltypes, synchronous );
     }
   if( generate )
     {
-    if( verbosity >= 0 &&
-        ( max_errors >= 0 || max_retries || o_direct || o_trunc ||
-          complete_only || nosplit || retrim || sparse || synchronous ) )
+    if( max_errors >= 0 || max_retries || o_direct || o_trunc ||
+        complete_only || nosplit || retrim || sparse || synchronous )
       show_error( "warning: options -C -d -D -e -n -r -R -S and -t are ignored in generate-logfile mode" );
 
-    return do_generate( ipos, opos, max_size, iname, oname, logname, cluster,
-                        hardbs, verbosity );
+    return do_generate( ipos, opos, domain, iname, oname, logname, cluster, hardbs );
     }
-  return do_rescue( ipos, opos, max_size, iname, oname, logname, cluster,
+  return do_rescue( ipos, opos, domain, iname, oname, logname, cluster,
                     hardbs, max_errors, max_retries, o_direct, o_trunc,
-                    verbosity, complete_only, nosplit, retrim, sparse,
-                    synchronous );
+                    complete_only, nosplit, retrim, sparse, synchronous );
   }
